'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import bpy, os
from bpy.props import StringProperty, BoolProperty
from ... import __package__ as base_package


class SRP_Addon_Prefs(bpy.types.AddonPreferences):
    bl_idname = base_package

    # Preset file path
    Render_Preset_File_Path: StringProperty(
        name= "渲染预设文件路径",
        description="Enter the path where you want to save your render presets",
        default='//',
        subtype='DIR_PATH')
    
 ########## Default Output Name #################

    # Checkbox for the Default-Name button
    Default_Name_Button: BoolProperty(
        name="启用默认名称按钮",
        description="If checked, there is a button to set a default  " \
                    "Export Name in   Properties > Output Properties > Output ",
        default=True)

    # Checkbox Create my own Default Output Name
    Manipulate_Default_Output_Name: BoolProperty(
        name="操纵默认名称",
        description="If checked, a default output name can be defined. " \
                    "This is optional, a good default name is already defined.",
        default=False)

    # Use .blend File Path Checkbox
    Default_Name_File_Path_Bool: BoolProperty(
        name="使用当前Blender文件路径",
        description="If checked, the path of the current blender file is used.  ",
        default=True)

    # Path
    Default_Name_File_Path: StringProperty(
        name= "默认导出路径",
        description="Here you can enter the default export path.",
        default='',
        subtype='DIR_PATH')
    
    # Use Main Folder Checkbox
    Default_Name_Main_Folder_Bool: BoolProperty(
        name="使用主文件夹",
        description="If checked, a main folder will be created.  ",
        default=True)

    # Main Folder
    Default_Name_Main_Folder: StringProperty(
        name= "主文件夹名称",
        description="Here you can enter the default main folder name.",
        default='Blender_Exports')

    # Use Filename Checkbox 1
    Default_Name_Sequence_Folder_Bool: BoolProperty(
        name="使用当前文件名",
        description="If checked, the file name is used.  "\
                    "Only available when exporting an image sequence.",
        default=True)
    Add_Word_Sequence_to_Sequence_Folder: BoolProperty(
        name="添加世界序列到序列文件夹",
        description="If checked, the word Sequence is added to the Sequence Folder",
        default=True
    )
        
    # Folder Name
    Default_Name_Sequence_Folder: StringProperty(
        name= "默认文件夹名称",
        description="Here you can enter the default sequence folder name. "\
                    "Only available when exporting an image sequence." ,
        default='')

    # Use Filename Checkbox 2
    Default_Name_Filename_Bool: BoolProperty(
        name="使用当前文件名",
        description="If checked, the file name is used.  ",
        default=True)
    # Filename
    Default_Name_Filename: StringProperty(
        name= "默认文件名",
        description="Here you can enter the default filename.",
        default='')


    # Set Default Output Name Butten on or off

    def draw(self, context):

        layout = self.layout

    # Preset File Path
        layout.use_property_split = True
        layout.use_property_decorate = True
        layout.prop(self,"Render_Preset_File_Path",text="预设文件路径")
##################### Default Name ###############################
    # Default Name Button
        layout.use_property_split = False
        first_main_box = layout.box()
        first_main_box.prop(self, "Default_Name_Button")
        first_main_box.prop(self, "Manipulate_Default_Output_Name")
        
        if self.Manipulate_Default_Output_Name:
            main_box = first_main_box.box()
            main_box.use_property_split = True
            main_box.label(text="默认导出名称设置")
            #Path
            box = main_box.box()
            col = box.column()
            col.prop(self,"Default_Name_File_Path_Bool")
            col = col.column()
            col.enabled = self.Default_Name_File_Path_Bool == False
            col.prop(self,"Default_Name_File_Path")
            # Main Folder
            box = main_box.box()
            col = box.column()
            col.prop(self,"Default_Name_Main_Folder_Bool")
            col = col.column()
            col.enabled = self.Default_Name_Main_Folder_Bool
            col.prop(self,"Default_Name_Main_Folder")
            # Sequence Folder
            box = main_box.box()
            col = box.column()
            col.prop(self,"Default_Name_Sequence_Folder_Bool",text="使用当前文件名作为序列文件夹")
            SequWordCol = col.column()
            SequWordCol.prop(self,"Add_Word_Sequence_to_Sequence_Folder", text="添加世界序列")
            col = col.column()
            col.enabled = self.Default_Name_Sequence_Folder_Bool == False
            col.prop(self,"Default_Name_Sequence_Folder")
            # Filename
            box = main_box.box()
            col = box.column()
            col.prop(self,"Default_Name_Filename_Bool")
            col = col.column()
            col.enabled = self.Default_Name_Filename_Bool == False
            col.prop(self,"Default_Name_Filename")