'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import bpy
from ..operator.SRP_Copy_Camera_Settings import SRP_OT_Copy_Camera_Settings
from ..operator.SRP_OT_Render_Presets import SRP_OT_Render_Presets
from ..operator.SRP_OT_Render_Settings_to_Text_Editor import SRP_OT_Render_Settings_to_Text_Editor
from ..operator.SRP_OT_Remove_Render_Preset import SRP_OT_Remove_Render_Preset
from ..operator.SRP_OT_Replace_Render_Preset import SRP_OT_Replace_Render_Preset
from ..operator.Enum_Functions.SRP_Enum_OP_Render_Settings import SRP_Enum_OP_Render_Settings

# Display into an existing panel
def Panel_Render_Presets(self, context):

    layout = self.layout
    srp_p = context.scene.srp_props # Get my bool props
    obj = context.active_object

    box = layout.box()
    box.label(text="Render Presets")
    box.prop(srp_p, "Include_Camera") # Set my bool-prop with the name "Include_Camera" into UI

    col = box.column(align=True)
    col.enabled = srp_p.Include_Camera
    col.prop(srp_p, "Load_Settings_to_all_Cameras")

    col = box.column()
    col.prop(srp_p, "SaveFrameRange")
    
    col = box.column()
    col.scale_y = 1
    
    if srp_p.Include_Camera and obj.type != 'CAMERA':
        col.label(text="Please select a camera.")
    else:
        col.label(text="")

    # Copy Camera Settings button
    box.operator(SRP_OT_Copy_Camera_Settings.bl_idname, icon= "OPTIONS")

    # Load Settings to Text Editor
    box.operator(SRP_OT_Render_Settings_to_Text_Editor.bl_idname, icon="COPYDOWN")
    
    box.separator(factor = 0.05) # make some room to the next UI elements

    SRP_Enum_OP_Render_Settings(context) # Update enum items before drawing them

    # Preset Menu 
    row = box.row(align=True) 
    row.scale_y = 1.3   
    if srp_p.Include_Camera:
        row.enabled = obj.type == 'CAMERA'
        row.prop(srp_p,'Enum_Render_Presets_Prop',text='')
        row.operator(SRP_OT_Render_Presets.bl_idname, text="", icon='ADD')
        row.operator(SRP_OT_Replace_Render_Preset.bl_idname, text="", icon='FILE_TICK')
        row.operator(SRP_OT_Remove_Render_Preset.bl_idname, text="", icon='REMOVE')
    else:
        row.prop(srp_p,'Enum_Render_Presets_Prop', text ='')
        row.operator(SRP_OT_Render_Presets.bl_idname, text="", icon='ADD')
        row.operator(SRP_OT_Replace_Render_Preset.bl_idname, text="", icon='FILE_TICK')
        row.operator(SRP_OT_Remove_Render_Preset.bl_idname, text="", icon='REMOVE')

    box.separator(factor = 1) # make some room to the next ui-elements

        
        