'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import bpy
from ..operator.Enum_Functions.SRP_Enum_OP_Render_Settings import SRP_Enum_OP_Render_Settings
from ..operator.Enum_Functions.SRP_Enum_OP_Execute_Render_Presets import SRP_Enum_OP_Execute_Render_Presets
from ..operator.Enum_Functions.SRP_Enum_OP_Execute_Cam_Presets import SRP_Enum_OP_Execute_Cam_Presets
from ..operator.Enum_Functions.SRP_Enum_OP_Camera_Settings import SRP_Enum_OP_Camera_Settings
from ..operator.Enum_Functions.SRP_Enum_OP_Render_Passes import SRP_Enum_OP_Render_Passes
from ..operator.Enum_Functions.SRP_Enum_OP_Execute_Render_Passes import SRP_Enum_OP_Execute_Render_Passes_Presets
from bpy.props import BoolProperty, EnumProperty


class SRP_Props(bpy.types.PropertyGroup):

    # Checkbox for Include Camera
    Include_Camera: BoolProperty(
        name="包括相机设置",
        description="If checked, the Camera Settings will be stored  " \
                    "and loaded with the Render-Settings, a camera needs " \
                    "to be selected",
        default=False,
        )

    # Checkbox for Copy Settings to all Cameras
    Load_Settings_to_all_Cameras: BoolProperty(
        name="将设置加载到所有摄像机",
        description="If checked, the Settings will be loaded to all Cameras " \
                    "in the current Scene. A camera must be selected ",
        default=False)

    # Checkbox for Frame Range
    SaveFrameRange: BoolProperty(
        name="保存帧范围",
        description="If checked, the Scene Frame Range will also be saved. " ,       
        default=False)

            
    Enum_Render_Presets_Prop : EnumProperty(name="保存渲染预设",
                                    items=lambda self, context: SRP_Enum_OP_Render_Settings.EnumItems,
                                    update=SRP_Enum_OP_Execute_Render_Presets,
                                    description="Choose your render preset. This menu is grayed out if Include Camera Settings \n" 
                                                "is activated above this menu but no camera is selected. \n"
                                                "In the add-on settings it needs a path to know where to store the presets, \n"
                                                "otherwise no presets can be created or used.",
                                        )

    SRP_Enum_OP_Camera_Settings.EnumItems = [] #important, error would say, there is no attribute in the function

    Enum_Camera_Presets_Prop : EnumProperty(name="保存相机预设",
                                    items=lambda self, context: SRP_Enum_OP_Camera_Settings.EnumItems,
                                    update=SRP_Enum_OP_Execute_Cam_Presets,  
                                    description="In the add-on settings it needs a path to know where to store the presets, \n"
                                                "otherwise no presets can be created or used.",                              
                                        )
    Enum_Render_Passes_Presets_Prop : EnumProperty(name="渲染过程预设",
                                    items=lambda self, context: SRP_Enum_OP_Render_Passes.EnumItems,
                                    update=SRP_Enum_OP_Execute_Render_Passes_Presets,   
                                    description="In the add-on settings it needs a path to know where to store the presets, \n"
                                                "otherwise no presets can be created or used.",                             
                                        )
    # This Bool is used to not execute a preset when a new one is created because it will be selected.
    Execute_Preset_Bool : BoolProperty(name="执行预设(bool)",
                                        default=True) 
    