'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''


import bpy 
from .Props import SRP_Props
from .Panel_Set_Default_Name import Panel_Set_Default_Name, Panel_Set_Default_Name_Nodes, Panel_Set_Default_Image_Name
from .Panel_Render_Preset import Panel_Render_Presets                    
from .Addon_Prefs import SRP_Addon_Prefs
from .Panel_Camera_Preset import Camera_Preset_Panel
from .Panel_Render_Passes import Render_Passes_Presets_Panel




classes = (
    SRP_Props,
    SRP_Addon_Prefs,
)

def register_menus():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)
    # Add Props
    bpy.types.Scene.srp_props = bpy.props.PointerProperty(type=SRP_Props)
    # Add to extisting Panels:
    bpy.types.RENDER_PT_output.prepend(Panel_Set_Default_Name)
    bpy.types.RENDER_PT_context.prepend(Panel_Render_Presets)
    bpy.types.DATA_PT_context_camera.prepend(Camera_Preset_Panel)
    bpy.types.NODE_PT_active_node_properties.prepend(Panel_Set_Default_Name_Nodes)
    bpy.types.VIEWLAYER_PT_layer.prepend(Render_Passes_Presets_Panel) # Passes in Eevee
    bpy.types.FILEBROWSER_PT_directory_path.append(Panel_Set_Default_Image_Name)

def unregister_menus():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)
    # Remove Props
    del bpy.types.Scene.srp_props
    # Add to extisting Panels:
    bpy.types.RENDER_PT_output.remove(Panel_Set_Default_Name)
    bpy.types.RENDER_PT_context.remove(Panel_Render_Presets)
    bpy.types.DATA_PT_context_camera.remove(Camera_Preset_Panel)
    bpy.types.NODE_PT_active_node_properties.remove(Panel_Set_Default_Name_Nodes)
    bpy.types.VIEWLAYER_PT_layer.remove(Render_Passes_Presets_Panel)
    bpy.types.FILEBROWSER_PT_directory_path.remove(Panel_Set_Default_Image_Name)

    