'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import bpy, random, pathlib
from .Function_Add_Subtract_Number import ExportNameNumber

class SRP_OT_Export_Name_Number_subtract_Nodes(bpy.types.Operator):
    ''' subtract number of the filename and the sequence folder'''

    bl_idname = "srp.export_name_number_subtract_nodes" # Name in API
    bl_label = "Export Name Number Subtract" # Name in searchbar
    bl_description = "Subtract a number to the filename and the sequence folder"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        
        # Get Node, get File Format
        act_node = context.scene.node_tree.nodes.active
        FileFormat = act_node.format.file_format

        # Get Filepath
        if FileFormat == 'OPEN_EXR_MULTILAYER':
            ExpPath = act_node.base_path
        else:
            ExpPath = act_node.base_path +'/'+ act_node.file_slots[0].path # Add filename to path for ExportNameNumber function
        
        ExpPath, filenumber = ExportNameNumber(ExportPath=ExpPath, FileFormat=FileFormat, Math='subtract') # Use my function
        
        # Set Path 
        FullPath = pathlib.Path(ExpPath)
        Parent_Folder = str(FullPath.parent)
        # Set full path with filename as base_path, if Open EXR Multilayer Format
        if FileFormat == 'OPEN_EXR_MULTILAYER':
            context.scene.node_tree.nodes.active.base_path = str(FullPath)
        else:
            # set filename (separate in the Output Node)
            context.scene.node_tree.nodes.active.base_path = Parent_Folder
            act_node = context.scene.node_tree.nodes.active
            act_node.file_slots[0].path = FullPath.stem

        # set node color
        act_node.use_custom_color = True
        act_node.color[0] = random.uniform(0,1)
        act_node.color[1] = random.uniform(0,1)
        act_node.color[2] = random.uniform(0,1)

        return{'FINISHED'}




