'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import bpy, pathlib
from ...menu.Utility import get_prefs
from .Function_Default_Output_Name import Function_Default_Output_Name

class SRP_OT_Single_Image_Name(bpy.types.Operator):

    bl_idname = "srp.single_image_name" # Name in API
    bl_label = "Set Default Output Name" # Name in searchbar
    bl_description = "Sets a default output name based on the blend file name \n " \
                        "Please note: No folders can be created by pressing this button. \n" \
                        "This button is intended to save individual images."
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        # get Add-on prefs
        Addon_Prefs = get_prefs()
        # get file format
        FileFormat = context.scene.render.image_settings.file_format
        # get fielpath parent
        FullPath = pathlib.Path(Function_Default_Output_Name(FileFormat=FileFormat))
        Export_Folder = str(FullPath.parent.parent)
        # set filepath parent
        context.screen.areas[0].spaces[0].params.directory = Export_Folder.encode()
        #get suffix
        preset_name = context.screen.areas[0].spaces[0].params.filename.split('.') 
        # set file name
        if len(preset_name)>1:
            Image_Name = FullPath.stem +'.' + preset_name[1] #add filename and suffix
        else:
            Image_Name = FullPath.stem  # add only filename if no suffix was there

        context.screen.areas[0].spaces[0].params.filename = Image_Name

        return {'FINISHED'}