'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import bpy
import pathlib
from mathutils import Vector # Needed for Safe Areas Settings (they are Vectors)
from bpy.props import StringProperty
from ..operator.Enum_Functions.SRP_Enum_OP_Camera_Settings import SRP_Enum_OP_Camera_Settings
from ..menu.Utility import get_prefs

class SRP_OT_Camera_Presets(bpy.types.Operator):
    """Add a new camera preset."""
    bl_idname = 'srp.save_camera_preset'
    bl_label = 'Save Camera Presets'
    bl_options = {'REGISTER', 'UNDO'}

    Camera_Preset_Name : StringProperty(name="Camera Preset Name",
                                        default="")

########## Camera Presets List ###############
    Camera_Presets= [ 
        "#################################################",
        "############## Camera Presets ###################",
        "#################################################",
        "-",
        " # Lens",
            "bpy.context.active_object.data.type",
            "bpy.context.active_object.data.panorama_type",
            "bpy.context.active_object.data.angle",
            "bpy.context.active_object.data.lens_unit",
            "bpy.context.active_object.data.shift_x",
            "bpy.context.active_object.data.shift_y",
            "bpy.context.active_object.data.clip_start",
            "bpy.context.active_object.data.clip_end",
            "bpy.context.active_object.data.ortho_scale",
            "bpy.context.active_object.data.fisheye_fov",
            "bpy.context.active_object.data.fisheye_lens", 
            "bpy.context.active_object.data.fisheye_fov", 
            "bpy.context.active_object.data.latitude_min", 
            "bpy.context.active_object.data.latitude_max", 
            "bpy.context.active_object.data.longitude_min", 
            "bpy.context.active_object.data.longitude_max", 
            "bpy.context.active_object.data.fisheye_fov",
            "bpy.context.active_object.data.fisheye_polynomial_k0",
            "bpy.context.active_object.data.fisheye_polynomial_k1",
            "bpy.context.active_object.data.fisheye_polynomial_k2",
            "bpy.context.active_object.data.fisheye_polynomial_k3",
            "bpy.context.active_object.data.fisheye_polynomial_k4",


        " # Camera",
            "bpy.context.active_object.data.sensor_fit",
            "bpy.context.active_object.data.sensor_width",
            "bpy.context.active_object.data.sensor_height",
        " # Safe Areas", #(they are shared between all cameras, so not necessary in "Copy_Camera_Settings")
            "bpy.context.active_object.data.show_safe_areas",
            "bpy.context.scene.safe_areas.title.x",
            "bpy.context.scene.safe_areas.title.y",
            "bpy.context.scene.safe_areas.action.x",
            "bpy.context.scene.safe_areas.action.y",
            "bpy.context.active_object.data.show_safe_center",
            "bpy.context.scene.safe_areas.title_center.x",
            "bpy.context.scene.safe_areas.title_center.y",
            "bpy.context.scene.safe_areas.action_center.x",
            "bpy.context.scene.safe_areas.action_center.y",
        " # Background Images",
            "bpy.context.active_object.data.show_background_images", #---- For now without image-Path and Settings
        "# Viewport Display",
            "bpy.context.active_object.data.display_size",
            "bpy.context.active_object.data.show_limits",
            "bpy.context.active_object.data.show_mist",
            "bpy.context.active_object.data.show_sensor",
            "bpy.context.active_object.data.show_name",
            "bpy.context.active_object.data.show_passepartout",
            "bpy.context.active_object.data.passepartout_alpha",
            "bpy.context.active_object.data.show_composition_thirds",
            "bpy.context.active_object.data.show_composition_center",
            "bpy.context.active_object.data.show_composition_center_diagonal",
            "bpy.context.active_object.data.show_composition_golden",
            "bpy.context.active_object.data.show_composition_golden_tria_a",
            "bpy.context.active_object.data.show_composition_golden_tria_b",
            "bpy.context.active_object.data.show_composition_harmony_tri_a",
            "bpy.context.active_object.data.show_composition_harmony_tri_b",
        "# Depth of Field",
            "bpy.context.active_object.data.dof.use_dof",
            "bpy.context.active_object.data.dof.focus_object",
            "bpy.context.active_object.data.dof.focus_distance",
            "bpy.context.active_object.data.dof.aperture_fstop",
            "bpy.context.active_object.data.dof.aperture_blades",
            "bpy.context.active_object.data.dof.aperture_rotation",
            "bpy.context.active_object.data.dof.aperture_ratio",
        "# Stereoscopy --> not available if Multyview disabled.",
            "bpy.context.active_object.data.stereo.convergence_mode",
            "bpy.context.active_object.data.stereo.convergence_distance",
            "bpy.context.active_object.data.stereo.interocular_distance",
            "bpy.context.active_object.data.stereo.use_spherical_stereo",
            "bpy.context.active_object.data.stereo.use_pole_merge",
            "bpy.context.active_object.data.stereo.pole_merge_angle_from",
            "bpy.context.active_object.data.stereo.pole_merge_angle_to",
            "bpy.context.active_object.data.stereo.pivot",
        ]
   
############## Execute ####################

    def execute(self, context):


        filename = self.Camera_Preset_Name
        if not filename[-3:] == ".py":
            filename = filename+ ".py"
            # replace empty space with an underline
        if " " in filename:
            filename = "_".join(filename.split())

        # get path from Addon Prefs and add filename
        Addon_Prefs = get_prefs()
        TruePath = Addon_Prefs.Render_Preset_File_Path
        # create directory "Camera_Settings" as parent folder
        pathlib.Path(TruePath+"camera_presets").mkdir(parents=True, exist_ok=True)
        # Add new folder to path
        TruePath = TruePath + '/camera_presets/'
        FilePath = pathlib.Path(TruePath+filename)

        # Write lines into file
        with open(FilePath, 'w') as f:
            f.write('import bpy')
            f.write('\n')
            f.write('\n')
            for line in self.Camera_Presets:
                if line[0] == "#": #Write Comments
                    f.write(line)
                    f.write('\n') 
                if line == "-":
                    f.write('\n')
                else:
                    try:  # Check Attribute
                        if hasattr(eval(line.rsplit('.',1)[0]),line.rsplit('.',1)[1]): 
                            Value = eval(line)
                            if type(Value) == str:
                                Value = "'"+Value+"'"
                        else:
                            print('This setting is not available in the Blender version used now: ',line)
                            continue
                        f.write(line)
                        f.write(' = ')
                        f.write(str(Value))
                        f.write('\n') # nächste Zeile
                    except:
                        pass
        
        #### Select new made Preset in the Enum ###
        SRP_Enum_OP_Camera_Settings(context)# Update items 
        srp_p = context.scene.srp_props
        Preset_files = pathlib.Path(TruePath).glob('*.py')
        Preset_Names = [f.name for f in Preset_files if f.is_file()] # get preset list for index
        for i in Preset_Names:
            if i == filename:
                srp_p.Enum_Camera_Presets_Prop = str(Preset_Names.index(i))

        # Clear Preset Name in the StringProperty
        self.Camera_Preset_Name = ''

        return{'FINISHED'}

############## Invoke #####################
    # This function and the StringProperty at the beginning creates a popup menu
    def invoke(self,context,evenet):
        return context.window_manager.invoke_props_dialog(self)
