import bpy
import pathlib
from bpy.props import StringProperty
from .SRP_OT_Camera_Presets import SRP_OT_Camera_Presets
from ..operator.Enum_Functions.SRP_Enum_OP_Render_Passes import SRP_Enum_OP_Render_Passes
from ..menu.Utility import get_prefs

class SRP_OT_Render_Passes_Preset(bpy.types.Operator):
    ''' Save all Render Passes into a file'''

    bl_idname = "srp.save_render_passes_preset" # Name in API
    bl_label = "Save Render Passes" # Name in searchbar
    bl_description = "Saves the current render passes to a file."
    bl_options = {'REGISTER', 'UNDO'}

    Render_Passes_Preset_Name : StringProperty(name="Render Preset Name",
                                        default="")

########## Render Preset List ###############

    Render_Passes = [
        "#################################################",
        "############## Render Passes ###################",
        "#################################################",
        "-",
        "##### Eevee #####",
        "-",
        "## Passes ##",
        "-",
        "# Data",
        "bpy.context.view_layer.use_pass_combined",
        "bpy.context.view_layer.use_pass_z",
        "bpy.context.view_layer.use_pass_mist",
        "bpy.context.view_layer.use_pass_normal",
        "# Light",
        "bpy.context.view_layer.use_pass_diffuse_direct",
        "bpy.context.view_layer.use_pass_diffuse_color",
        "bpy.context.view_layer.use_pass_glossy_direct",
        "bpy.context.view_layer.use_pass_glossy_color",
        "bpy.context.view_layer.eevee.use_pass_volume_direct",
        "bpy.context.view_layer.use_pass_volume_direct",
        "bpy.context.view_layer.use_pass_emit",
        "bpy.context.view_layer.use_pass_environment",
        "bpy.context.view_layer.use_pass_shadow",
        "bpy.context.view_layer.use_pass_ambient_occlusion",
        "# Effects",
        "bpy.context.view_layer.eevee.use_pass_bloom",
        "bpy.context.view_layer.eevee.use_pass_transparent",
        "# Cryptomatte",
        "bpy.context.view_layer.use_pass_cryptomatte_object",
        "bpy.context.view_layer.use_pass_cryptomatte_material",
        "bpy.context.view_layer.use_pass_cryptomatte_asset",
        "bpy.context.view_layer.pass_cryptomatte_depth",
        "bpy.context.view_layer.use_pass_cryptomatte_accurate",
        "-",
        "###### Cycles #######",
        # Some Settings are in Eevee and Cycles for a good overview in the script
        "-",
        "## Passes ##",
        "-",
        "# Data",
        "bpy.context.view_layer.use_pass_combined",
        "bpy.context.view_layer.use_pass_z",
        "bpy.context.view_layer.use_pass_mist",
        "bpy.context.view_layer.use_pass_position",
        "bpy.context.view_layer.use_pass_normal",
        "bpy.context.view_layer.use_pass_vector",
        "bpy.context.view_layer.use_pass_uv",
        "bpy.context.view_layer.cycles.denoising_store_passes",
        "bpy.context.view_layer.use_pass_object_index",
        "bpy.context.view_layer.use_pass_material_index",
        "bpy.context.view_layer.cycles.pass_debug_sample_count",
        "bpy.context.view_layer.pass_alpha_threshold",
        "# Light",
        "bpy.context.view_layer.use_pass_diffuse_direct",
        "bpy.context.view_layer.use_pass_diffuse_indirect",
        "bpy.context.view_layer.use_pass_diffuse_color",
        "bpy.context.view_layer.use_pass_glossy_direct",
        "bpy.context.view_layer.use_pass_glossy_indirect",
        "bpy.context.view_layer.use_pass_glossy_color",
        "bpy.context.view_layer.use_pass_transmission_direct",
        "bpy.context.view_layer.use_pass_transmission_indirect",
        "bpy.context.view_layer.use_pass_transmission_color",
        "bpy.context.view_layer.cycles.use_pass_volume_direct",
        "bpy.context.view_layer.cycles.use_pass_volume_indirect",
        "bpy.context.view_layer.use_pass_emit",
        "bpy.context.view_layer.use_pass_environment",
        "bpy.context.view_layer.use_pass_shadow",
        "bpy.context.view_layer.use_pass_ambient_occlusion",
        "bpy.context.view_layer.cycles.use_pass_shadow_catcher",
        "# Cryptomatte",
        "bpy.context.view_layer.use_pass_cryptomatte_object",
        "bpy.context.view_layer.use_pass_cryptomatte_material",
        "bpy.context.view_layer.use_pass_cryptomatte_asset",
        "bpy.context.view_layer.pass_cryptomatte_depth",
        "# Filter",
        "bpy.context.view_layer.use_sky",
        "bpy.context.view_layer.use_solid",
        "bpy.context.view_layer.use_strand",
        "bpy.context.view_layer.use_volumes",
        "bpy.context.view_layer.use_motion_blur",
        "bpy.context.view_layer.cycles.use_denoising",
        # Hydra Storm has no own passes
        # Material override makes no sense to integrate.

        ]
    
############## Execute ####################

    def execute(self, context):

        srp_p = context.scene.srp_props
        Render_Passes = self.Render_Passes

        Presets = []
        Presets.extend(Render_Passes)

        filename = self.Render_Passes_Preset_Name
        if not filename[-3:] == ".py":
            filename = filename+ ".py"
            # replace empty space with an underline
        if " " in filename:
            filename = "_".join(filename.split())
            
        # get path from Addon Prefs and add filename
        Addon_Prefs = get_prefs()
        TruePath = Addon_Prefs.Render_Preset_File_Path

        # create directory "Render_Passes" as parent folder
        pathlib.Path(TruePath+"render_passes").mkdir(parents=True, exist_ok=True)
        # Add new folder to path
        TruePath = TruePath + '/render_passes/'

        FilePath = pathlib.Path(TruePath+filename)

        # Write lines into file
        with open(FilePath, 'w') as f:
            f.write('import bpy')
            f.write('\n')
            f.write('\n')
            for line in Presets:
                if line[0] == "#": #Write Comments
                    f.write(line)
                    f.write('\n') 
                if line == "-":
                    f.write('\n')
                else:
                    try:  # Check Attribute
                        if hasattr(eval(line.rsplit('.',1)[0]),line.rsplit('.',1)[1]): 
                            Value = eval(line)
                            if type(Value) == str:
                                Value = "'"+Value+"'"
                        else:
                            continue
                        f.write(line)
                        f.write(' = ')
                        f.write(str(Value))
                        f.write('\n') # nächste Zeile
                    except:
                        pass
        
        #### Select new made Preset in the Enum ###
        SRP_Enum_OP_Render_Passes(context) # Update enum items
        srp_p.Execute_Preset_Bool = False # Don't execute because Load Settings to all Cameras could be unwanted although activ
        Preset_files = pathlib.Path(TruePath).glob('*.py')
        Preset_Names = [f.name for f in Preset_files if f.is_file()] # get preset list for index
        for i in Preset_Names:
            if i == filename:
                srp_p.Enum_Render_Passes_Presets_Prop = str(Preset_Names.index(i))
        srp_p.Execute_Preset_Bool = True # Set it back to True to execute selected Presets

        # Clear Preset Name in the StringProperty
        self.Render_Passes_Preset_Name = ''

        return{'FINISHED'}

############## Invoke #####################
    # This function and the StringProperty at the beginning creates a popup menu
    def invoke(self,context,evenet):
        return context.window_manager.invoke_props_dialog(self)