import bpy
import pathlib
from bpy.props import StringProperty
from .SRP_OT_Camera_Presets import SRP_OT_Camera_Presets
from ..operator.Enum_Functions.SRP_Enum_OP_Render_Settings import SRP_Enum_OP_Render_Settings
from ..menu.Utility import get_prefs

class SRP_OT_Render_Presets(bpy.types.Operator):
    ''' Save all Rendersettings into a file'''

    bl_idname = "srp.save_render_preset" # Name in API
    bl_label = "Save Render Settings" # Name in searchbar
    bl_description = "Saves the current render settings to a file.This menu is grayed out if Include Camera Settings is activated above this menu but no camera is selected."
    bl_options = {'REGISTER', 'UNDO'}

    Render_Preset_Name : StringProperty(name="Render Preset Name",
                                        default="")

    # menu is greyed out if no camera is selected
    @classmethod
    def poll(cls, context): 
        srp_p = context.scene.srp_props
        if srp_p.Include_Camera and not context.active_object.type == 'CAMERA':
            return False
        return True

########## Render Preset List ###############

    Render_Presets = [
        "#################################################",
        "############## Render Presets ###################",
        "#################################################",
        "-",
        "##### Preferences #####",
            "bpy.context.preferences.addons['cycles'].preferences.compute_device_type",
        "# Enable Hydra Storm render engine Add-on",
            "addon_utils.enable('hydra_storm')",
        "##### Render-Settings #####",
            "bpy.context.scene.render.engine",
            "bpy.context.scene.render.use_stamp",
        "# Performance",
            "bpy.context.scene.render.use_high_quality_normals",
            "bpy.context.scene.render.hair_type",
            "bpy.context.scene.render.hair_subdiv",
        "# Film",
            "bpy.context.scene.render.filter_size",
            "bpy.context.scene.render.film_transparent",
       " # Simplify",
            "bpy.context.scene.render.use_simplify",
            "bpy.context.scene.render.simplify_subdivision",
            "bpy.context.scene.render.simplify_child_particles",
            "bpy.context.scene.render.simplify_volumes",
            "bpy.context.scene.render.simplify_subdivision_render",
            "bpy.context.scene.render.simplify_child_particles_render",
            "bpy.context.scene.render.simplify_gpencil",
            "bpy.context.scene.render.simplify_gpencil_onplay",
            "bpy.context.scene.render.simplify_gpencil_view_fill",
            "bpy.context.scene.render.simplify_gpencil_modifier",
            "bpy.context.scene.render.simplify_gpencil_shader_fx",
            "bpy.context.scene.render.simplify_gpencil_tint",
            "bpy.context.scene.render.simplify_gpencil_antialiasing",
        "# Freestyle",
            "bpy.context.scene.render.use_freestyle",
            "bpy.context.scene.render.line_thickness_mode",
            "bpy.context.scene.render.line_thickness",
        "# Color Management",
            "bpy.context.scene.display_settings.display_device",
            "bpy.context.scene.view_settings.view_transform",
            "bpy.context.scene.view_settings.look",
            "bpy.context.scene.view_settings.exposure",
            "bpy.context.scene.view_settings.gamma",
            "bpy.context.scene.sequencer_colorspace_settings.name",
        "##### EEVEE-Settings #####",
        "# Sampling",
            "bpy.context.scene.eevee.taa_render_samples",
            "bpy.context.scene.eevee.taa_samples",
            "bpy.context.scene.eevee.use_taa_reprojection",
        "# Clamping",
            "bpy.context.scene.eevee.clamp_surface_direct",
            "bpy.context.scene.eevee.clamp_surface_indirect",
            "bpy.context.scene.eevee.clamp_volume_direct",
            "bpy.context.scene.eevee.clamp_volume_indirect",
        "# Raytracing",
            "bpy.context.scene.eevee.use_raytracing",
            "bpy.context.scene.eevee.ray_tracing_method",
            "bpy.context.scene.eevee.ray_tracing_options.resolution_scale",
            "bpy.context.scene.eevee.ray_tracing_options.trace_max_roughness",
            "bpy.context.scene.eevee.ray_tracing_options.screen_trace_quality",
            "bpy.context.scene.eevee.ray_tracing_options.screen_trace_thickness",
            "bpy.context.scene.eevee.ray_tracing_options.use_denoise",
            "bpy.context.scene.eevee.ray_tracing_options.denoise_spatial",
            "bpy.context.scene.eevee.ray_tracing_options.denoise_temporal",
            "bpy.context.scene.eevee.ray_tracing_options.denoise_bilateral",
            "bpy.context.scene.eevee.fast_gi_method",
            "bpy.context.scene.eevee.fast_gi_resolution",
            "bpy.context.scene.eevee.fast_gi_ray_count",
            "bpy.context.scene.eevee.fast_gi_step_count",
            "bpy.context.scene.eevee.fast_gi_quality",
            "bpy.context.scene.eevee.fast_gi_distance",
            "bpy.context.scene.eevee.fast_gi_thickness_near",
            "bpy.context.scene.eevee.fast_gi_thickness_far",
            "bpy.context.scene.eevee.fast_gi_bias",
        "# Ambient Occlusion",
            "bpy.context.scene.eevee.use_gtao",
            "bpy.context.scene.eevee.gtao_distance",
            "bpy.context.scene.eevee.gtao_factor",
            "bpy.context.scene.eevee.gtao_quality",
            "bpy.context.scene.eevee.use_gtao_bent_normals",
            "bpy.context.scene.eevee.use_gtao_bounce",
        "# Bloom",
            "bpy.context.scene.eevee.use_bloom",
            "bpy.context.scene.eevee.bloom_threshold",
            "bpy.context.scene.eevee.bloom_knee",
            "bpy.context.scene.eevee.bloom_radius",
            "bpy.context.scene.eevee.bloom_color.r",
            "bpy.context.scene.eevee.bloom_color.g",
            "bpy.context.scene.eevee.bloom_color.b",
            "bpy.context.scene.eevee.bloom_intensity",
            "bpy.context.scene.eevee.bloom_clamp",
        "# Depth of Field",
            "bpy.context.scene.eevee.bokeh_threshold",
            "bpy.context.scene.eevee.bokeh_neighbor_max",
            "bpy.context.scene.eevee.bokeh_denoise_fac",
            "bpy.context.scene.eevee.use_bokeh_high_quality_slight_defocus",
            "bpy.context.scene.eevee.use_bokeh_jittered",
            "bpy.context.scene.eevee.bokeh_overblur",
            "bpy.context.scene.eevee.bokeh_max_size",
       " # Subsurface Scattering",
            "bpy.context.scene.eevee.sss_samples",
            "bpy.context.scene.eevee.sss_jitter_threshold",
        "# Screen Space Reflection",
            "bpy.context.scene.eevee.use_ssr",
            "bpy.context.scene.eevee.use_ssr_halfres",
            "bpy.context.scene.eevee.use_ssr_refraction",
            "bpy.context.scene.eevee.ssr_quality",
            "bpy.context.scene.eevee.ssr_max_roughness",
            "bpy.context.scene.eevee.ssr_thickness",
            "bpy.context.scene.eevee.ssr_border_fade",
            "bpy.context.scene.eevee.ssr_firefly_fac",
        "# Motion Blur",
            "bpy.context.scene.eevee.use_motion_blur",
            "bpy.context.scene.eevee.motion_blur_position",
            "bpy.context.scene.eevee.motion_blur_shutter",
            "bpy.context.scene.eevee.motion_blur_depth_scale",
            "bpy.context.scene.eevee.motion_blur_max",
            "bpy.context.scene.eevee.motion_blur_steps",
            "bpy.context.scene.render.use_motion_blur",
            "bpy.context.scene.render.motion_blur_position", # Ab blender 4.2 cycles und Eevee motion blur in scene.render
            "bpy.context.scene.render.motion_blur_shutter",
            "bpy.context.scene.render.motion_blur_depth_scale",
            "bpy.context.scene.render.motion_blur_max",
            "bpy.context.scene.render.motion_blur_steps",
        "# Voluminetrics",
            "bpy.context.scene.eevee.volumetric_start",
            "bpy.context.scene.eevee.volumetric_end",
            "bpy.context.scene.eevee.volumetric_tile_size",
            "bpy.context.scene.eevee.volumetric_samples",
            "bpy.context.scene.eevee.volumetric_sample_distribution",
            "bpy.context.scene.eevee.use_volumetric_lights",
            "bpy.context.scene.eevee.volumetric_light_clamp",
            "bpy.context.scene.eevee.use_volumetric_shadows",
            "bpy.context.scene.eevee.volumetric_shadow_samples",
        "# Shadows",
            "bpy.context.scene.eevee.shadow_cube_size",
            "bpy.context.scene.eevee.shadow_cascade_size",
            "bpy.context.scene.eevee.use_shadow_high_bitdepth",
            "bpy.context.scene.eevee.use_soft_shadows",
            "bpy.context.scene.eevee.light_threshold",
        "# Indirect Lightning",
            "bpy.context.scene.eevee.gi_auto_bake",
            "bpy.context.scene.eevee.gi_diffuse_bounces",
            "bpy.context.scene.eevee.gi_cubemap_resolution",
            "bpy.context.scene.eevee.gi_visibility_resolution",
            "bpy.context.scene.eevee.gi_irradiance_smoothing",
            "bpy.context.scene.eevee.gi_glossy_clamp",
            "bpy.context.scene.eevee.gi_filter_quality",
            "bpy.context.scene.eevee.gi_cubemap_display_size",
            "bpy.context.scene.eevee.gi_irradiance_display_size",
            "bpy.context.scene.eevee.gi_show_cubemaps",
            "bpy.context.scene.eevee.gi_show_irradiance",
        "##### CYCLES-Settings #####",
            "bpy.context.scene.cycles.feature_set",
            "bpy.context.scene.cycles.device",
            "bpy.context.scene.cycles.shading_system",
        "# Sampling ",
            "bpy.context.scene.cycles.progressive",
            "bpy.context.scene.cycles.samples",
            "bpy.context.scene.cycles.preview_samples",
            "bpy.context.scene.cycles.use_adaptive_sampling",
            "bpy.context.scene.cycles.adaptive_threshold",
            "bpy.context.scene.cycles.adaptive_min_samples",
            "bpy.context.scene.cycles.use_preview_adaptive_sampling",
            "bpy.context.scene.cycles.preview_adaptive_threshold",
            "bpy.context.scene.cycles.preview_adaptive_min_samples",
            "bpy.context.scene.cycles.use_preview_denoising",
            "bpy.context.scene.cycles.preview_denoiser",
            "bpy.context.scene.cycles.preview_denoising_input_passes",
            "bpy.context.scene.cycles.preview_denoising_start_sample",
            "bpy.context.scene.cycles.time_limit",
            "bpy.context.scene.cycles.use_denoising",
            "bpy.context.scene.cycles.denoiser",
            "bpy.context.scene.cycles.denoising_input_passes",
            "bpy.context.scene.cycles.denoising_prefilter",
            "bpy.context.scene.cycles.seed",
            "bpy.context.scene.cycles.sample_offset",
            "bpy.context.scene.cycles.use_animated_seed",
            "bpy.context.scene.cycles.sampling_pattern",
            "bpy.context.scene.cycles.scrambling_distance",
            "bpy.context.scene.cycles.adaptive_scrambling_distance",
            "bpy.context.scene.cycles.preview_scrambling_distance",
            "bpy.context.scene.cycles.auto_scrambling_distance",
            "bpy.context.scene.cycles.min_light_bounces",
            "bpy.context.scene.cycles.min_transparent_bounces",
            "bpy.context.scene.cycles.use_light_tree",
            "bpy.context.scene.cycles.light_sampling_threshold",
        "# Denoising ",
            "bpy.context.scene.cycles.use_denoising",
            "bpy.context.scene.cycles.denoiser",
            "bpy.context.scene.cycles.use_preview_denoising",
            "bpy.context.scene.cycles.preview_denoiser",
            "bpy.context.scene.cycles.preview_denoising_start_sample",
            "bpy.context.scene.cycles.preview_denoising_input_passes",
        "# Path Guiding",
            "bpy.context.scene.cycles.use_guiding",
            "bpy.context.scene.cycles.guiding_training_samples",
            "bpy.context.scene.cycles.use_surface_guiding",
            "bpy.context.scene.cycles.use_volume_guiding",
        "# Advanced ",
            "bpy.context.scene.cycles.seed",
            "bpy.context.scene.cycles.use_animated_seed",
            "bpy.context.scene.cycles.sample_offset",
            "bpy.context.scene.cycles.sampling_pattern",
            "bpy.context.scene.cycles.auto_scrambling_distance",
            "bpy.context.scene.cycles.preview_scrambling_distance",
            "bpy.context.scene.cycles.scrambling_distance",
            "bpy.context.scene.cycles.use_square_samples",
            "bpy.context.scene.cycles.min_light_bounces",
            "bpy.context.scene.cycles.min_transparent_bounces",
            "bpy.context.scene.cycles.light_sampling_threshold",
            "bpy.context.scene.cycles.diffuse_samples",
            "bpy.context.scene.cycles.glossy_samples",
            "bpy.context.scene.cycles.transmission_samples",
            "bpy.context.scene.cycles.ao_samples",
            "bpy.context.scene.cycles.mesh_light_samples",
            "bpy.context.scene.cycles.subsurface_samples",
            "bpy.context.scene.cycles.volume_samples",
        "# Light Paths",
            "bpy.context.scene.cycles.max_bounces",
            "bpy.context.scene.cycles.diffuse_bounces",
            "bpy.context.scene.cycles.glossy_bounces",
            "bpy.context.scene.cycles.transparent_max_bounces",
            "bpy.context.scene.cycles.transmission_bounces",
            "bpy.context.scene.cycles.volume_bounces",
            "bpy.context.scene.cycles.sample_clamp_direct",
            "bpy.context.scene.cycles.sample_clamp_indirect",
            "bpy.context.scene.cycles.blur_glossy",
            "bpy.context.scene.cycles.caustics_reflective",
            "bpy.context.scene.cycles.caustics_refractive",
            "bpy.context.scene.cycles.use_fast_gi",
            "bpy.context.scene.world.light_settings.ao_factor",
            "bpy.context.scene.world.light_settings.distance", 
        "# Volumes ",
            "bpy.context.scene.cycles.volume_step_rate",
            "bpy.context.scene.cycles.volume_preview_step_rate",
            "bpy.context.scene.cycles.volume_max_steps",
        "# Subdivision",
            "bpy.context.scene.cycles.dicing_rate",
            "bpy.context.scene.cycles.preview_dicing_rate",
            "bpy.context.scene.cycles.offscreen_dicing_scale",
            "bpy.context.scene.cycles.max_subdivisions",
            "bpy.context.scene.cycles.dicing_camera",
        "# Hair",
            "bpy.context.scene.cycles_curves.shape",
            "bpy.context.scene.cycles_curves.subdivisions",
        "# Simplify",
            "bpy.context.scene.render.use_simplify",
            "bpy.context.scene.render.simplify_subdivision",
            "bpy.context.scene.render.simplify_child_particles",
            "bpy.context.scene.cycles.texture_limit",
            "bpy.context.scene.cycles.ao_bounces",
            "bpy.context.scene.render.simplify_volumes",
            "bpy.context.scene.render.simplify_subdivision_render",
            "bpy.context.scene.render.simplify_child_particles_render",
            "bpy.context.scene.cycles.texture_limit_render",
            "bpy.context.scene.cycles.ao_bounces_render",
        "# Motion Blur",
            "bpy.context.scene.render.use_motion_blur",
            "bpy.context.scene.cycles.motion_blur_position",
            "bpy.context.scene.render.motion_blur_shutter",
            "bpy.context.scene.cycles.rolling_shutter_type",
            "bpy.context.scene.cycles.rolling_shutter_duration",
        #                                   ---------- Motion blur Shutter Curve can't be stored for now
        "# Film",
            "bpy.context.scene.cycles.film_exposure",
            "bpy.context.scene.cycles.pixel_filter_type",
            "bpy.context.scene.cycles.filter_width",
            "bpy.context.scene.render.film_transparent",
            "bpy.context.scene.cycles.film_transparent_glass",
            "bpy.context.scene.cycles.film_transparent_roughness",
        "# Performance",
            "bpy.context.scene.render.threads_mode",
            "bpy.context.scene.render.threads",
            "bpy.context.scene.cycles.tile_size",
            "bpy.context.scene.render.tile_x",
            "bpy.context.scene.render.tile_y", 
            "bpy.context.scene.cycles.tile_order",
            "bpy.context.scene.cycles.use_progressive_refine",
            "bpy.context.scene.cycles.debug_use_spatial_splits",
            "bpy.context.scene.cycles.debug_use_compact_bvh",
            "bpy.context.scene.cycles.debug_use_hair_bvh",
            "bpy.context.scene.cycles.debug_bvh_time_steps",
            "bpy.context.scene.render.use_save_buffers",
            "bpy.context.scene.render.use_persistent_data",
            "bpy.context.scene.render.preview_pixel_size",
            "bpy.context.scene.cycles.preview_start_resolution",
        "# Bake",
            "bpy.context.scene.render.use_bake_multires",
            "bpy.context.scene.cycles.bake_type",
            "bpy.context.scene.render.bake.use_pass_direct",
            "bpy.context.scene.render.bake.use_pass_indirect",
            "bpy.context.scene.render.bake.use_pass_diffuse",
            "bpy.context.scene.render.bake.use_pass_glossy",
            "bpy.context.scene.render.bake.use_pass_transmission",
            "bpy.context.scene.render.bake.use_pass_ambient_occlusion",
            "bpy.context.scene.render.bake.use_pass_emit",
            "bpy.context.scene.render.bake.use_selected_to_active",
            "bpy.context.scene.render.bake.use_cage",
            "bpy.context.scene.render.bake.cage_object",
            "bpy.context.scene.render.bake.cage_extrusion",
            "bpy.context.scene.render.bake.max_ray_distance",
            "bpy.context.scene.render.bake.margin",
            "bpy.context.scene.render.bake.margin_type",
            "bpy.context.scene.render.bake.use_clear",
        "# Grease Pencil",
            "bpy.context.scene.grease_pencil_settings.antialias_threshold",
       " # Freestyle",
            "bpy.context.scene.render.line_thickness_mode",
            "bpy.context.scene.render.line_thickness",
        "##### WORKBENCH ##########",
            "bpy.context.scene.grease_pencil_settings.antialias_threshold",
            "bpy.context.scene.render.use_high_quality_normals",
        "# Sampling",
            "bpy.context.scene.display.render_aa",
            "bpy.context.scene.display.viewport_aa",
        "# Lighting",
            "bpy.context.scene.display.shading.light",
            "bpy.context.scene.display.shading.studio_light",
            "bpy.context.scene.display.shading.use_world_space_lighting",
            "bpy.context.scene.display.shading.studiolight_rotate_z",
        "# Color",
            "bpy.context.scene.display.shading.color_type",
        "# Options",
            "bpy.context.scene.display.shading.show_backface_culling",
            "bpy.context.scene.display.shading.show_xray",
            "bpy.context.scene.display.shading.xray_alpha",
            "bpy.context.scene.display.shading.show_shadows",
            "bpy.context.scene.display.shading.shadow_intensity",
            "bpy.context.scene.display.light_direction.x",
            "bpy.context.scene.display.light_direction.y",
            "bpy.context.scene.display.light_direction.z",
            "bpy.context.scene.display.shadow_shift",
            "bpy.context.scene.display.shadow_focus",
            "bpy.context.scene.display.shading.show_cavity",
            "bpy.context.scene.display.shading.cavity_type",
            "bpy.context.scene.display.shading.cavity_ridge_factor",
            "bpy.context.scene.display.shading.cavity_valley_factor",
            "bpy.context.scene.display.matcap_ssao_samples",
            "bpy.context.scene.display.matcap_ssao_distance",
            "bpy.context.scene.display.matcap_ssao_attenuation",
            "bpy.context.scene.display.shading.curvature_ridge_factor",
            "bpy.context.scene.display.shading.curvature_valley_factor",
            "bpy.context.scene.display.shading.use_dof",
            "bpy.context.scene.display.shading.show_object_outline",
            "bpy.context.scene.display.shading.object_outline_color.r",
            "bpy.context.scene.display.shading.object_outline_color.g",
            "bpy.context.scene.display.shading.object_outline_color.b",
            "bpy.context.scene.display.shading.show_specular_highlight",
        "######## Hydra Storm render engine #########",
        "# Quality",
            "bpy.context.scene.hydra_storm.viewport.max_lights",
            "bpy.context.scene.hydra_storm.viewport.use_tiny_prim_culling",
            "bpy.context.scene.hydra_storm.final.max_lights",
            "bpy.context.scene.hydra_storm.final.use_tiny_prim_culling",
        "# Volumes",
            "bpy.context.scene.hydra_storm.viewport.volume_raymarching_step_size",
            "bpy.context.scene.hydra_storm.viewport.volume_raymarching_step_size_lighting",
            "bpy.context.scene.hydra_storm.viewport.volume_max_texture_memory_per_field",
            "bpy.context.scene.hydra_storm.final.volume_raymarching_step_size",
            "bpy.context.scene.hydra_storm.final.volume_raymarching_step_size_lighting",
            "bpy.context.scene.hydra_storm.final.volume_max_texture_memory_per_field",
        "# Hydra Debug",
            "bpy.context.scene.hydra.export_method",
        "# Film",
            "bpy.context.scene.render.film_transparent",
        "##### OUTPUT PROPERTIES #####",
            "bpy.context.scene.render.resolution_x",
            "bpy.context.scene.render.resolution_y",
            "bpy.context.scene.render.resolution_percentage",
            "bpy.context.scene.render.pixel_aspect_x",
            "bpy.context.scene.render.pixel_aspect_y",
            "bpy.context.scene.render.use_border",
            "bpy.context.scene.render.use_crop_to_border",
            "bpy.context.scene.frame_start",
            "bpy.context.scene.frame_end",
            "bpy.context.scene.frame_step",
            "bpy.context.scene.render.fps",
            "bpy.context.scene.render.frame_map_old",
            "bpy.context.scene.render.frame_map_new",
        "# Stereoscopy",
            "bpy.context.scene.render.use_multiview",  
            "bpy.context.scene.render.views_format",
            "bpy.context.scene.render.views['left'].use", #----for now, more added Multifiew-outputs "use" will be ignored
            "bpy.context.scene.render.views['right'].use",
            "bpy.context.scene.render.views['left'].file_suffix",
            "bpy.context.scene.render.views['right'].file_suffix",
        "# Output",
        "# Images + Video",
            "bpy.context.scene.render.use_file_extension",
            "bpy.context.scene.render.use_render_cache",
            "bpy.context.scene.render.image_settings.file_format",
            "bpy.context.scene.render.image_settings.color_mode",
            "bpy.context.scene.render.image_settings.views_format",
            "bpy.context.scene.render.image_settings.compression",
            "bpy.context.scene.render.image_settings.quality",
            "bpy.context.scene.render.image_settings.jpeg2k_codec",
            "bpy.context.scene.render.image_settings.use_jpeg2k_cinema_preset",
            "bpy.context.scene.render.image_settings.use_jpeg2k_cinema_48",
            "bpy.context.scene.render.image_settings.use_jpeg2k_ycc",
            "bpy.context.scene.render.image_settings.use_cineon_log",
            "bpy.context.scene.render.image_settings.color_depth",
            "bpy.context.scene.render.image_settings.exr_codec",
            "bpy.context.scene.render.image_settings.use_zbuffer",
            "bpy.context.scene.render.image_settings.use_preview",
            "bpy.context.scene.render.use_overwrite",
            "bpy.context.scene.render.use_placeholder",
            "bpy.context.scene.render.image_settings.tiff_codec",
        "# FFmpeg Video",
            "bpy.context.scene.render.ffmpeg.format",
            "bpy.context.scene.render.ffmpeg.use_autosplit",
            "bpy.context.scene.render.ffmpeg.codec",
            "bpy.context.scene.render.ffmpeg.constant_rate_factor",
            "bpy.context.scene.render.ffmpeg.ffmpeg_preset",
            "bpy.context.scene.render.ffmpeg.gopsize",
            "bpy.context.scene.render.ffmpeg.use_max_b_frames",
            "bpy.context.scene.render.ffmpeg.max_b_frames",
            "bpy.context.scene.render.ffmpeg.audio_codec",
            "bpy.context.scene.render.ffmpeg.audio_channels",
            "bpy.context.scene.render.ffmpeg.audio_mixrate",
            "bpy.context.scene.render.ffmpeg.audio_bitrate",
            "bpy.context.scene.render.ffmpeg.audio_volume",
            "bpy.context.scene.render.ffmpeg.video_bitrate",
            "bpy.context.scene.render.ffmpeg.minrate",
            "bpy.context.scene.render.ffmpeg.maxrate",
            "bpy.context.scene.render.ffmpeg.buffersize",
            "bpy.context.scene.render.ffmpeg.muxrate",
            "bpy.context.scene.render.ffmpeg.packetsize",
            "bpy.context.scene.render.ffmpeg.use_lossless_output",
        "# Metadata",
            "bpy.context.scene.render.metadata_input",
            "bpy.context.scene.render.use_stamp_date",
            "bpy.context.scene.render.use_stamp_time",
            "bpy.context.scene.render.use_stamp_render_time",
            "bpy.context.scene.render.use_stamp_frame",
            "bpy.context.scene.render.use_stamp_frame_range",
            "bpy.context.scene.render.use_stamp_memory",
            "bpy.context.scene.render.use_stamp_hostname",
            "bpy.context.scene.render.use_stamp_camera",
            "bpy.context.scene.render.use_stamp_lens",
            "bpy.context.scene.render.use_stamp_scene",
            "bpy.context.scene.render.use_stamp_marker",
            "bpy.context.scene.render.use_stamp_filename",
        "# Post Processing",
            "bpy.context.scene.render.use_compositing",
            "bpy.context.scene.render.use_sequencer",
            "bpy.context.scene.render.dither_intensity",
            "-",
            "-",

        ]
    
############## Execute ####################

    def execute(self, context):

        srp_p = context.scene.srp_props
        Render_Presets = self.Render_Presets

        Presets = []
        Presets.extend(Render_Presets)

        # Delete Frame Range from Render_Presets if wanted
        if not srp_p.SaveFrameRange:
            Presets.remove("bpy.context.scene.frame_start")
            Presets.remove("bpy.context.scene.frame_end")
        if not context.scene.render.engine == 'HYDRA_STORM':
            Presets.remove("# Enable Hydra Storm render engine Add-on")
            Presets.remove("addon_utils.enable('hydra_storm')")

        filename = self.Render_Preset_Name
        if not filename[-3:] == ".py":
            filename = filename+ ".py"
            # replace empty space with an underline
        if " " in filename:
            filename = "_".join(filename.split())
            
        # get path from Addon Prefs and add filename
        Addon_Prefs = get_prefs()
        TruePath = Addon_Prefs.Render_Preset_File_Path
        # create directory "Render_Settings" or "Render_Presets_and_Camera_Presets" as parent folder
        
        if srp_p.Include_Camera:
            pathlib.Path(TruePath+"render_presets_and_camera_presets").mkdir(parents=True, exist_ok=True)
            ###### Import Camera Settings if Inclusive Camera Settings is enabled ######
            Presets.extend(SRP_OT_Camera_Presets.Camera_Presets)
            # Add new folder to path
            TruePath = TruePath + '/render_presets_and_camera_presets/'
        else:
            pathlib.Path(TruePath+"render_presets").mkdir(parents=True, exist_ok=True)
            # Add new folder to path
            TruePath = TruePath + '/render_presets/'

        FilePath = pathlib.Path(TruePath+filename)

        # Write lines into file
        with open(FilePath, 'w') as f:
            f.write('import bpy, addon_utils')
            f.write('\n')
            f.write('\n')
            for line in Presets:
                if line[0] == "#": #Write Comments
                    f.write(line)
                    f.write('\n') 
                if line == "addon_utils.enable('hydra_storm')":
                    f.write(line)
                    f.write('\n')
                if line == "-":
                    f.write('\n')
                else:
                    try:  # Check Attribute
                        if hasattr(eval(line.rsplit('.',1)[0]),line.rsplit('.',1)[1]): 
                            Value = eval(line)
                            if type(Value) == str:
                                Value = "'"+Value+"'"
                        else:
                            print('This setting is not available in the Blender version used now: ',line)
                            continue
                        f.write(line)
                        f.write(' = ')
                        f.write(str(Value))
                        f.write('\n') # nächste Zeile
                    except:
                        pass
        
        #### Select new made Preset in the Enum ###
        SRP_Enum_OP_Render_Settings(context) # Update enum items
        srp_p.Execute_Preset_Bool = False # Don't execute because Load Settings to all Cameras could be unwanted although activ
        Preset_files = pathlib.Path(TruePath).glob('*.py')
        Preset_Names = [f.name for f in Preset_files if f.is_file()] # get preset list for index
        for i in Preset_Names:
            if i == filename:
                srp_p.Enum_Render_Presets_Prop = str(Preset_Names.index(i))
        srp_p.Execute_Preset_Bool = True # Set it back to True to execute selected Presets

        # Clear Preset Name in the StringProperty
        self.Render_Preset_Name = ''

        return{'FINISHED'}

############## Invoke #####################
    # This function and the StringProperty at the beginning creates a popup menu
    def invoke(self,context,evenet):
        return context.window_manager.invoke_props_dialog(self)