'''
Copyright (C) 2021 Zensurist
andib@gmx.ch

Created by Andi Bösch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import bpy
from .SRP_OT_Camera_Presets import SRP_OT_Camera_Presets
from .SRP_OT_Render_Presets import SRP_OT_Render_Presets

class SRP_OT_Render_Settings_to_Text_Editor(bpy.types.Operator):
    ''' Load Current Settings to Text Editor'''

    bl_idname = "srp.render_settings_to_text_editor" # Name in API
    bl_label = "将设置渲染到文本编辑器" # Name in searchbar
    bl_description = "Show current render settings in the Text Editor"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context): 
        srp_p = context.scene.srp_props
        if srp_p.Include_Camera and context.active_object != None:
            if context.active_object.type != 'CAMERA': 
                return False
        return True

    def execute(self, context):

        Presets = []

        Render_Presets = SRP_OT_Render_Presets.Render_Presets
        Camera_Presets = SRP_OT_Camera_Presets.Camera_Presets

        Presets.extend(Render_Presets)

        srp_p = context.scene.srp_props
    
    # Manipulate the RenderPreset List
        if not srp_p.SaveFrameRange:
                Presets.remove("bpy.context.scene.frame_start")
                Presets.remove("bpy.context.scene.frame_end")
                
        if not context.scene.render.engine == 'HYDRA_STORM':
            Presets.remove("# Enable Hydra Storm render engine Add-on")
            Presets.remove("addon_utils.enable('hydra_storm')")

        if srp_p.Include_Camera:
            Text_File = bpy.data.texts.new('SRP_Render_Settings_and_Camera_Settings.py')
            Presets.extend(Camera_Presets)
            
        else:
            Text_File = bpy.data.texts.new('SRP_Render_Settings.py')
        
    ##### Write text to Text Editor #####
        Text_File.write('import bpy, addon_utils')
        Text_File.write('\n')
        Text_File.write('\n')
        for line in Presets:
            if line[0] == "#": #Write Comments
                Text_File.write(line)
                Text_File.write('\n') 
            if line == "addon_utils.enable('hydra_storm')":
                Text_File.write(line)
                Text_File.write('\n')
            if line == "-":
                Text_File.write('\n')
            else:
                try:  # Check Attribute
                    if hasattr(eval(line.rsplit('.',1)[0]),line.rsplit('.',1)[1]): 
                        Value = eval(line)
                        if type(Value) == str:
                            Value = "'"+Value+"'"
                    else:
                        continue
                    Text_File.write(line)
                    Text_File.write(' = ')
                    Text_File.write(str(Value))
                    Text_File.write('\n') # nächste Zeile
                except:
                    pass

        self.report({'INFO'}, "Settings in the text drop down menu in the Text Editor window")
            
        return{'FINISHED'}

        



